"""
check server stability
"""

import aiohttp
import asyncio
from .exceptions import APITimeoutError, APIConnectionError

async def check_connection(url: str):
    
    timeout = aiohttp.ClientTimeout(total=5)
    try:
        async with aiohttp.ClientSession(timeout=timeout) as session:
            async with session.head(f'{url}/') as response:
                response.raise_for_status()
    except asyncio.TimeoutError:
        raise APITimeoutError("Connection timed out")
    except aiohttp.ClientConnectorError as e:
        raise APIConnectionError(f"Connection error to url {url}/: {e}")
    except aiohttp.ClientResponseError as e:
        if e.status >= 404:
            pass
        else:
            raise APIConnectionError(f"HTTP error {e.status} for url {url}/: {e.message}")
    except aiohttp.ClientError as e:
        raise APIConnectionError(f"Client error for url {url}/: {e}")
    return True


"""
(GET) /users/online method implementation.
"""

import aiohttp
import asyncio
from .exceptions import APITimeoutError, APIConnectionError

async def get_online(url: str):
    timeout = aiohttp.ClientTimeout(total=5)
    try:
        async with aiohttp.ClientSession(timeout=timeout) as session:
            async with session.get(f"{url}/users/online") as response:
                response.raise_for_status()
                data = await response.json()
                return data
    except asyncio.TimeoutError:
        raise APITimeoutError("Connection timed out")
    except aiohttp.ClientConnectorError as e:
        raise APIConnectionError(f"Connection error to url {url}/users/online: {e}")
    except aiohttp.ClientResponseError as e:
        raise APIConnectionError(f"HTTP error {e.status} for url {url}/users/online: {e.message}")
    except aiohttp.ClientError as e:
        raise APIConnectionError(f"Client error for url {url}/users/online: {e}")
    return None

async def get_raw_online(url: str):
    timeout = aiohttp.ClientTimeout(total=5)
    try:
        async with aiohttp.ClientSession(timeout=timeout) as session:
            async with session.get(f"{url}/users/rawOnline") as response:
                response.raise_for_status()
                data = await response.text()
                return data
    except asyncio.TimeoutError:
        raise APITimeoutError("Connection timed out")
    except aiohttp.ClientConnectorError as e:
        raise APIConnectionError(f"Connection error to url {url}/users/rawOnline: {e}")
    except aiohttp.ClientResponseError as e:
        raise APIConnectionError(f"HTTP error {e.status} for url {url}/users/rawOnline: {e.message}")
    except aiohttp.ClientError as e:
        raise APIConnectionError(f"Client error for url {url}/users/rawOnline: {e}")
    return None


import aiohttp
import asyncio
from .exceptions import APITimeoutError, APIConnectionError

async def send_announce(url: str, message: str):
    timeout = aiohttp.ClientTimeout(total=5)
    try:
        async with aiohttp.ClientSession(timeout=timeout) as session:
            async with session.post(f"{url}/users/announce", json={"message": message}) as response:
                response.raise_for_status()
                data = await response.json()
                return data
    except asyncio.TimeoutError:
        raise APITimeoutError("Connection timed out")
    except aiohttp.ClientConnectorError as e:
        raise APIConnectionError(f"Connection error to url {url}/users/announce: {e}")
    except aiohttp.ClientResponseError as e:
        if e.status == 400:
            raise APIConnectionError(f"Bad Request for url {url}/users/announce: {e.message}. Are you sure what admin account is enabled in config.js?")
        else:
            raise APIConnectionError(f"HTTP error {e.status} for url {url}/users/announce: {e.message}")
    except aiohttp.ClientError as e:
        raise APIConnectionError(f"Client error for url {url}/users/announce: {e}")
    return None

async def send_mail_to_all(url: str, message: str):
    timeout = aiohttp.ClientTimeout(total=5)
    try:
        async with aiohttp.ClientSession(timeout=timeout) as session:
            async with session.post(f"{url}/users/sendMailToAll", json={"message": message}) as response:
                response.raise_for_status()
                data = await response.json()
                return data
    except asyncio.TimeoutError:
        raise APITimeoutError("Connection timed out")
    except aiohttp.ClientConnectorError as e:
        raise APIConnectionError(f"Connection error to url {url}/users/sendMailToAll: {e}")
    except aiohttp.ClientResponseError as e:
        raise APIConnectionError(f"HTTP error {e.status} for url {url}/users/sendMailToAll: {e.message}")
        


# entry point for aiomrim command 
def hello():
    print("Welcome to aiomrim! This is the async SDK for MRIM-Server (Renaissance). Version: v1.0\nOur GitHub repo: https://github.com/fayzetwin1/aiomrim\nContact with me: contact@fayzetwin.xyz\nRenaissance (MRIM-Server) repo: https://github.com/mrimsu/mrim-server\n\nMade with love by fayzetwin! <3")


"""
Working with account-related functionalities (not API implementation)
"""
import aiomysql
from typing import Any, List, Tuple
import hashlib

async def register_account(
    login: str, nickname: str, firstname: str, sex: int, password: str, # default values
    host: str, port: int, user = "root", db_password = "", database = "mrimdb", # mysql connection
    last_name = None, birthday = None, zodiac = None, phone = None, avatar = None,
    create_default_groups: bool = True) -> int: # optional value
    
    DEFAULT_GROUPS: List[str] = [
        " ^t ^` ^c    ^l ^o",
        "        ^k  ",
        " ^z            ",
        " ^~ ^a ^b     ^l    ^k  ",
    ]

    hash = hashlib.md5(password.encode("utf-8")).hexdigest()
    db = {"host": host, "port": port, "user": user, "password": db_password, "db": database}
    user_data = {
        "login": login, "nick": nickname, "f_name": firstname, "l_name": last_name, 
        "sex": sex, "passwd": hash, "birthday": birthday, "zodiac": zodiac, "phone": phone, "avatar": avatar}
    
    fields, values, placeholders = [], [], []

    for k, v in user_data.items():
        if v is not None:
            fields.append(k)
            values.append(v)
            placeholders.append("%s")

    sql_user = f"INSERT INTO user ({', '.join(fields)}) VALUES ({', '.join(placeholders)})"

    values_tuple = tuple(values)

    user_id: int = -1
    pool = None

    try:
        pool = await aiomysql.create_pool(**db)
        async with pool.acquire() as conn:
            async with conn.cursor() as cursor:
                await cursor.execute(sql_user, values_tuple)
                user_id = cursor.lastrowid
                
                if not user_id:
                    raise Exception("failed to retrieve user_id after insertion")
                
                if create_default_groups:
                    sql_group = "INSERT INTO contact_group (user_id, name, idx) VALUES (%s, %s, %s)"
                    group_data: List[Tuple[int, str, int]] = [
                        (user_id, name, idx) 
                        for idx, name in enumerate(DEFAULT_GROUPS)
                    ]
                    print(f'sql_group: {sql_group}, group_data: {group_data}')
                    await cursor.executemany(sql_group, group_data)

                await conn.commit()
        return user_id
    except Exception as e:
        raise e
    finally:
        if pool:
            pool.close()
            await pool.wait_closed()
    
    
    


class APIConnectionError(Exception):
    """Exception raised for errors in the API connection."""
    pass

class APITimeoutError(Exception):
    """Exception raised for API timeout errors."""
    pass

class APIResponseError(Exception):
    """Exception raised for invalid API responses."""
    def __init__(self, message, status_code):
        super().__init__(message)
        self.status_code = status_code



from setuptools import setup, find_packages

setup(
    name="aiomrim",
    version="1.0",
    packages=find_packages(),
    install_requires=[
        "aiohttp >= 3.12.0",
        "asyncio >= 3.4.3",
        "aiomysql >= 0.2.0",
    ],
    entry_points={
        'console_scripts': [
            'aiomrim=aiomrim:hello',
        ],
    },
    author="fayzetwin",
    author_email="contact@fayzetwin.xyz",
    description="Awesome SDK for Async Operations with MRIM-Server (https://github.com/mrimsu/mrim-server)",
    url="https://github.com/fayzetwin1/aiomrim")